searchData={"items":[{"type":"module","title":"Circuits.I2C","doc":"`Circuits.I2C` lets you communicate with hardware devices using the I2C\nprotocol.","ref":"Circuits.I2C.html"},{"type":"type","title":"Circuits.I2C.address/0","doc":"I2C device address\n\nThis is a \"7-bit\" address for the device. Some devices specify an \"8-bit\"\naddress in their documentation. You can tell if you have an \"8-bit\" address\nif it's greater than 127 (0x7f) or if the documentation talks about different\nread and write addresses. If you have an 8-bit address, divide it by 2.","ref":"Circuits.I2C.html#t:address/0"},{"type":"type","title":"Circuits.I2C.backend/0","doc":"Backends specify an implementation of a Circuits.I2C.Backend behaviour\n\nThe second parameter of the Backend 2-tuple is a list of options. These are\npassed to the behaviour function call implementations.","ref":"Circuits.I2C.html#t:backend/0"},{"type":"type","title":"Circuits.I2C.bus/0","doc":"Connection to a real or virtual I2C controller","ref":"Circuits.I2C.html#t:bus/0"},{"type":"function","title":"Circuits.I2C.bus_names/0","doc":"Return a list of available I2C bus names\n\nIf nothing is returned, it's possible that the kernel driver for that I2C bus\nis not enabled or the kernel's device tree is not configured. On Raspbian,\nrun `raspi-config` and look in the advanced options.\n\n```elixir\niex> Circuits.I2C.bus_names()\n[\"i2c-1\"]\n```","ref":"Circuits.I2C.html#bus_names/0"},{"type":"function","title":"Circuits.I2C.close/1","doc":"Close the I2C bus","ref":"Circuits.I2C.html#close/1"},{"type":"function","title":"Circuits.I2C.detect_devices/0","doc":"Convenience method to scan all I2C buses for devices\n\nThis is only intended to be called from the IEx prompt. Programs should\nuse `detect_devices/1`.","ref":"Circuits.I2C.html#detect_devices/0"},{"type":"function","title":"Circuits.I2C.detect_devices/1","doc":"Scan the I2C bus for devices\n\nSince there's no official way of enumerating I2C devices, this attempts to\nfigure out what's on the bus by seeing what devices respond.\n\n```elixir\niex> Circuits.I2C.detect_devices(\"i2c-1\")\n[4]\n```\n\nThe return value is a list of device addresses that were detected on the\nspecified I2C bus. If you get back `'Hh'` or other letters, then IEx\nconverted the list to an Erlang string. Run `i v()` to get information about\nthe return value and look at the raw string representation for addresses.\n\n> #### Warning {: .warning}\n> This is intended to be a debugging aid. Reading bytes from devices can\n> advance internal state machines and might cause them to get out of sync\n> with other code.\n\nIf you already have opened an I2C bus, then pass that to `detect_devices/1`\ninstead of the string name.","ref":"Circuits.I2C.html#detect_devices/1"},{"type":"function","title":"Circuits.I2C.device_present?/2","doc":"Return whether a device is present\n\nThis function performs a simplistic check for an I2C device on the specified\nbus and address. It's not perfect, but works enough to be useful. Be warned\nthat it does perform an I2C read on the specified address and this may cause\nsome devices to actually do something.","ref":"Circuits.I2C.html#device_present?/2"},{"type":"function","title":"Circuits.I2C.discover/2","doc":"Scan all I2C buses for one or more devices\n\nThis function takes a list of possible addresses and an optional detection\nfunction. It only scans addresses in the possible addresses list to avoid\ndisturbing unrelated I2C devices.\n\nIf a detection function is not passed in, a default one that performs a\nsimple read and checks whether it succeeds is used. If the desired device has\nan ID register or other means of identification, the optional function should\ntry to query that. If passing a custom function, be sure to return `false`\nrather than raise if there are errors.\n\nA list of bus name and address tuples is returned. The list may be empty.\n\nSee also `discover_one/2`.","ref":"Circuits.I2C.html#discover/2"},{"type":"function","title":"Circuits.I2C.discover_one/2","doc":"Scans all I2C buses for one specific device\n\nThis function and `discover_one!/2` are convenience functions for the use\ncase of helping a user find a specific device. They both call `discover/2` with\na list of possible I2C addresses and an optional function for checking whether\nthe device is present.\n\nThis function returns an `:ok` or `:error` tuple depending on whether one and\nonly one device was found. See `discover_one!/2` for the raising version.","ref":"Circuits.I2C.html#discover_one/2"},{"type":"function","title":"Circuits.I2C.discover_one!/2","doc":"Same as `discover_one/2` but raises on error","ref":"Circuits.I2C.html#discover_one!/2"},{"type":"function","title":"Circuits.I2C.info/1","doc":"Return info about the low level I2C interface\n\nThis may be helpful when debugging I2C issues.","ref":"Circuits.I2C.html#info/1"},{"type":"function","title":"Circuits.I2C.open/2","doc":"Open an I2C bus\n\nI2C bus names depend on the platform. Names are of the form \"i2c-n\" where the\n\"n\" is the bus number.  The correct bus number can be found in the\ndocumentation for the device or on a schematic. Another option is to call\n`Circuits.I2C.bus_names/0` to list them for you.\n\nThe same I2C bus may be opened more than once. There is no need to share\nit between modules.\n\nOn success, this returns a `Circuits.I2C.Bus.t()` struct for accessing the\nI2C bus. Use this in subsequent calls to read and write I2C devices.\n\nOptions depend on the backend. The following are for the I2CDev (default)\nbackend:\n\n* `:retries` - the number of times to retry a transaction. I.e. 2 retries means\n  the transaction is attempted at most 3 times. Defaults to 0 retries.\n* `:timeout` - the time in milliseconds to wait for a transaction to complete\n  or <0 to avoid setting it. Defaults to -1.","ref":"Circuits.I2C.html#open/2"},{"type":"type","title":"Circuits.I2C.open_options/0","doc":"I2C open options\n\nSee I2C backend documentation, device driver caveats, and function calls in this\nmodule for notes. In general,\n\n* `:retries` - the number of times to retry a transaction. I.e. 2 retries means\n  the transaction is attempted at most 3 times. Defaults to 0 retries.\n* `:timeout` - the time in milliseconds to wait for a transaction to complete.\n  Any value <0 means to use the device driver or hardware default. The default\n  value is -1 to avoid setting it, but it is very common for the default to be 1000 ms.","ref":"Circuits.I2C.html#t:open_options/0"},{"type":"type","title":"Circuits.I2C.present?/0","doc":"Function to report back whether a device is present\n\nSee `discover/2` for how a custom function can improve device detection when\nthe type of device being looked for is known.","ref":"Circuits.I2C.html#t:present?/0"},{"type":"function","title":"Circuits.I2C.read/4","doc":"Initiate a read transaction to the I2C device at the specified `address`\n\nSee `t:transfer_options/0` for options, but backend may provide more.","ref":"Circuits.I2C.html#read/4"},{"type":"function","title":"Circuits.I2C.read!/4","doc":"Initiate a read transaction and raise on error","ref":"Circuits.I2C.html#read!/4"},{"type":"type","title":"Circuits.I2C.transfer_options/0","doc":"I2C transfer options\n\nSee I2C backend documentation, device driver caveats, and function calls in this\nmodule for notes. In general,\n\n* `:retries` - override the number of times to retry a transaction from what was\n  passed to `open/3`.","ref":"Circuits.I2C.html#t:transfer_options/0"},{"type":"function","title":"Circuits.I2C.write/4","doc":"Write `data` to the I2C device at `address`.\n\nSee `t:transfer_options/0` for options, but backend may provide more.","ref":"Circuits.I2C.html#write/4"},{"type":"function","title":"Circuits.I2C.write!/4","doc":"Write `data` to the I2C device at `address` and raise on error\n\nSee `t:transfer_options/0` for options, but backend may provide more.","ref":"Circuits.I2C.html#write!/4"},{"type":"function","title":"Circuits.I2C.write_read/5","doc":"Write `data` to an I2C device and then immediately issue a read.\n\nThis function is useful for devices that want you to write the \"register\"\nlocation to them first and then issue a read to get its contents. Many\ndevices operate this way and this function will issue the commands\nback-to-back on the I2C bus. Some I2C devices actually require that the read\nimmediately follows the write. If the target supports this, the I2C\ntransaction will be issued that way. On the Raspberry Pi, this can be enabled\nglobally with `File.write!(\"/sys/module/i2c_bcm2708/parameters/combined\", \"1\")`\n\nSee `t:transfer_options/0` for options, but backend may provide more.","ref":"Circuits.I2C.html#write_read/5"},{"type":"function","title":"Circuits.I2C.write_read!/5","doc":"Write `data` to an I2C device and then immediately issue a read. Raise on errors.\n\nSee `t:transfer_options/0` for options, but backend may provide more.","ref":"Circuits.I2C.html#write_read!/5"},{"type":"behaviour","title":"Circuits.I2C.Backend","doc":"Backends provide the connection to the real or virtual I2C controller","ref":"Circuits.I2C.Backend.html"},{"type":"callback","title":"Circuits.I2C.Backend.bus_names/1","doc":"Return the I2C bus names on this system\n\nSee backend documentation for options.","ref":"Circuits.I2C.Backend.html#c:bus_names/1"},{"type":"callback","title":"Circuits.I2C.Backend.info/0","doc":"Return information about this backend","ref":"Circuits.I2C.Backend.html#c:info/0"},{"type":"callback","title":"Circuits.I2C.Backend.open/2","doc":"Open an I2C bus\n\nBus names are typically of the form `\"i2c-n\"` and available buses may be\nfound by calling `bus_names/1`.\n\nSee `t:Circuits.I2C.open_options/0` for guidance on options.","ref":"Circuits.I2C.Backend.html#c:open/2"},{"type":"type","title":"Circuits.I2C.Backend.transfer_options/0","doc":"I2C transfer options\n\nSupport for options is backend-specific. Backends are encouraged to\nimplement the following:\n\n* `:retries` - the number of retries for this transaction","ref":"Circuits.I2C.Backend.html#t:transfer_options/0"},{"type":"protocol","title":"Circuits.I2C.Bus","doc":"A bus is the connection to a real or virtual I2C controller","ref":"Circuits.I2C.Bus.html"},{"type":"function","title":"Circuits.I2C.Bus.close/1","doc":"Free up resources associated with the bus\n\nWell behaved backends free up their resources with the help of the Erlang garbage collector. However, it is good\npractice for users to call `Circuits.I2C.close/1` (and hence this function) so that\nlimited resources are freed before they're needed again.","ref":"Circuits.I2C.Bus.html#close/1"},{"type":"type","title":"Circuits.I2C.Bus.flag/0","doc":"I2C controller flags\n\n* `:supports_empty_write` - the controller supports sending empty binaries\n  to devices. These can be used for device detection.","ref":"Circuits.I2C.Bus.html#t:flag/0"},{"type":"function","title":"Circuits.I2C.Bus.flags/1","doc":"","ref":"Circuits.I2C.Bus.html#flags/1"},{"type":"function","title":"Circuits.I2C.Bus.read/4","doc":"Read data over I2C\n\nThe controller should try to read the specified number of bytes over I2C.\nIf the retry option is passed and non-zero, the transaction only needs to\nbe retried if there's an error. This means that fewer that the requested\nnumber of bytes may be returned.\n\nSee the implementation for options","ref":"Circuits.I2C.Bus.html#read/4"},{"type":"type","title":"Circuits.I2C.Bus.t/0","doc":"All the types that implement this protocol.","ref":"Circuits.I2C.Bus.html#t:t/0"},{"type":"function","title":"Circuits.I2C.Bus.write/4","doc":"Write data over I2C\n\nThe controller should write the passed in data to the specified I2C address.","ref":"Circuits.I2C.Bus.html#write/4"},{"type":"function","title":"Circuits.I2C.Bus.write_read/5","doc":"Write data and read a result in one I2C transaction\n\nThis function handles the common task of writing a register number\nto a device and then reading its contents. The controller should perform it\nas one transaction without a stop condition between the write and read.","ref":"Circuits.I2C.Bus.html#write_read/5"},{"type":"module","title":"Circuits.I2C.I2CDev","doc":"Circuits.I2C backend for the Linux i2c-dev interface\n\nThis backend works on Nerves, embedded Linux, and desktop Linux.","ref":"Circuits.I2C.I2CDev.html"},{"type":"function","title":"Circuits.I2C.I2CDev.bus_names/1","doc":"Return the I2C bus names on this system\n\nNo supported options","ref":"Circuits.I2C.I2CDev.html#bus_names/1"},{"type":"function","title":"Circuits.I2C.I2CDev.info/0","doc":"Return information about this backend","ref":"Circuits.I2C.I2CDev.html#info/0"},{"type":"function","title":"Circuits.I2C.I2CDev.open/2","doc":"Open an I2C bus\n\nBus names are typically of the form `\"i2c-n\"` and available buses may be\nfound by calling `Circuits.I2C.bus_names/0`.\n\nOptions:\n\n* `:retries` - the number of times to retry a transaction. I.e. 2 retries means\n  the transaction is attempted at most 3 times. Defaults to 0 retries.\n* `:timeout` - the time in milliseconds to wait for a transaction to complete.\n  Any value <0 means to use the device driver default which is probably 1000 ms.","ref":"Circuits.I2C.I2CDev.html#open/2"},{"type":"module","title":"Circuits.I2C.NilBackend","doc":"Circuits.I2C backend when nothing else is available","ref":"Circuits.I2C.NilBackend.html"},{"type":"function","title":"Circuits.I2C.NilBackend.bus_names/1","doc":"Return the I2C bus names on this system\n\nNo supported options","ref":"Circuits.I2C.NilBackend.html#bus_names/1"},{"type":"function","title":"Circuits.I2C.NilBackend.info/0","doc":"Return information about this backend","ref":"Circuits.I2C.NilBackend.html#info/0"},{"type":"function","title":"Circuits.I2C.NilBackend.open/2","doc":"Open an I2C bus\n\nNo supported options.","ref":"Circuits.I2C.NilBackend.html#open/2"},{"type":"extras","title":"Circuits.I2C","doc":"# Circuits.I2C\n\n[![Hex version](https://img.shields.io/hexpm/v/circuits_i2c.svg \"Hex version\")](https://hex.pm/packages/circuits_i2c)\n[![API docs](https://img.shields.io/hexpm/v/circuits_i2c.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/circuits_i2c/Circuits.I2C.html)\n[![CircleCI](https://circleci.com/gh/elixir-circuits/circuits_i2c.svg?style=svg)](https://circleci.com/gh/elixir-circuits/circuits_i2c)\n[![REUSE status](https://api.reuse.software/badge/github.com/elixir-circuits/circuits_i2c)](https://api.reuse.software/info/github.com/elixir-circuits/circuits_i2c)\n\n`Circuits.I2C` lets you communicate with hardware devices using the I2C protocol.\n\n*This is the v2.0 branch. Circuits.I2C v1.x is still maintained in the [maint-v1.x branch](https://github.com/elixir-circuits/circuits_i2c/tree/maint-v1.x).*\n\n`Circuits.I2C` v2.0  is an almost backwards compatible update to `Circuits.I2C`\nv1.x. Here's what's new:\n\n* Linux or Nerves are no longer required. In fact, the NIF supporting them won't\n  be compiled if you don't want it.\n* Develop using simulated I2C devices with\n  [CircuitsSim](https://github.com/elixir-circuits/circuits_sim)\n* Use USB->I2C adapters for development on your laptop (Coming soon)\n\nIf you've used `Circuits.I2C` v1.x, nearly all of your code will be the same. If\nyou're a library author, we'd appreciate if you could try this out and update\nyour `:circuits_i2c` dependency to allow v2.0. Details can be found in our\n[porting guide](PORTING.md).","ref":"readme.html"},{"type":"extras","title":"Getting started on Nerves and Linux - Circuits.I2C","doc":"By default, `Circuits.I2C` supports the Linux-based I2C driver interface so the\nfollowing instructions assume a Linux-based system like Nerves, Raspberry Pi OS,\nembedded Linux or even desktop Linux if I2C lines are exposed. If you want to\nuse `Circuits.I2C` on a different platform and support is available, generally\nthe only difference is to change the \"open\" call. The rest is the same.\n\nFirst off, add `circuits_i2c` to your `mix.exs`'s dependency list like any other\nElixir library:\n\n```elixir\ndef deps do\n  [{:circuits_i2c, \"~> 2.0\"}]\nend\n```\n\n`Circuits.I2C` doesn't load device drivers, so you may need to load them\nbeforehand. If you are using Nerves on a supported platform, this is enabled for\nyou already. If using Raspberry Pi OS, the [Adafruit Raspberry Pi I2C\ninstructions](https://learn.adafruit.com/adafruits-raspberry-pi-lesson-4-gpio-setup/configuring-i2c)\nmay be helpful.\n\nInternally, it uses the [Linux \"i2c-dev\"\ninterface](https://elixir.bootlin.com/linux/latest/source/Documentation/i2c/dev-interface)\nso that it does not require board-dependent code.","ref":"readme.html#getting-started-on-nerves-and-linux"},{"type":"extras","title":"Getting started without hardware - Circuits.I2C","doc":"If you don't have any real I2C devices, it's possible to work with simulated\ndevices. See the [CircuitsSim](https://github.com/elixir-circuits/circuits_sim)\nproject for details.","ref":"readme.html#getting-started-without-hardware"},{"type":"extras","title":"I2C background - Circuits.I2C","doc":"An [Inter-Integrated Circuit](https://en.wikipedia.org/wiki/I%C2%B2C) (I2C) bus\nsupports addressing hardware components and bidirectional use of the data line.\n\nThe following shows a bus IO expander connected via I2C to the processor.\n\n![I2C schematic](assets/images/schematic-i2c.png)\n\nThe protocol for talking to the IO expander is described in the [MCP23008\nDatasheet](http://www.microchip.com/wwwproducts/Devices.aspx?product=MCP23008).\nHere's a simple example of using it.\n\n```elixir\n# On the Raspberry Pi, the IO expander is connected to I2C bus 1 (i2c-1).\n# Its 7-bit address is 0x20. (see datasheet)\niex> alias Circuits.I2C\nCircuits.I2C\niex> {:ok, ref} = I2C.open(\"i2c-1\")\n{:ok, #Reference<...>}\n\n# By default, all 8 GPIOs are set to inputs. Set the 4 high bits to outputs\n# so that we can toggle the LEDs. (Write 0x0f to register 0x00)\niex> I2C.write(ref, 0x20, <<0x00, 0x0f>>)\n:ok\n\n# Turn on the LED attached to bit 4 on the expander. (Write 0x10 to register\n# 0x09)\niex> I2C.write(ref, 0x20, <<0x09, 0x10>>)\n:ok\n\n# Read all 11 of the expander's registers to see that the bit 0 switch is\n# the only one on and that the bit 4 LED is on.\niex> I2C.write(ref, 0x20, <<0>>)  # Set the next register to be read to 0\n:ok\n\niex> I2C.read(ref, 0x20, 11)\n{:ok, <<15, 0, 0, 0, 0, 0, 0, 0, 0, 17, 16>>}\n\n# The operation of writing one or more bytes to select a register and\n# then reading is very common, so a shortcut is to just run the following:\niex> I2C.write_read(ref, 0x20, <<0>>, 11)\n{:ok, <<15, 0, 0, 0, 0, 0, 0, 0, 0, 17, 16>>}\n\n# The 17 in register 9 says that bits 0 and bit 4 are high\n# We could have just read register 9.\n\niex> I2C.write_read(ref, 0x20, <<9>>, 1)\n{:ok, <<17>>}\n```","ref":"readme.html#i2c-background"},{"type":"extras","title":"Creating a new backend - Circuits.I2C","doc":"`Circuits.I2C` supports alternative backends to support non-Linux hardware,\ntesting, and simulation. A backend can support communication on more than one\nI2C bus.\n\nTo create a new backend, you need to implement the `Circuits.I2C.Backend`\nbehaviour. `Circuits.I2C` calls the `bus_names/1` callback to discover what I2C\nbuses are available and then it calls the `open/2` callback to use the I2C bus.","ref":"readme.html#creating-a-new-backend"},{"type":"extras","title":"FAQ - Circuits.I2C","doc":"","ref":"readme.html#faq"},{"type":"extras","title":"How do I debug? - Circuits.I2C","doc":"The most common issue is communicating with an I2C for the first time.  For I2C,\nfirst check that an I2C bus is available:\n\n```elixir\niex> Circuits.I2C.bus_names\n[\"i2c-1\"]\n```\n\nIf the list is empty, then I2C is either not available, not enabled, or not\nconfigured in the kernel. If you're using Raspbian, run `raspi-config` and check\nthat I2C is enabled in the advanced options. If you're on a BeagleBone, try\nrunning `config-pin` and see the [Universal I/O\nproject](https://github.com/cdsteinkuehler/beaglebone-universal-io) to enable\nthe I2C pins. On other ARM boards, double check that I2C is enabled in the\nkernel and that the device tree configures it.\n\nOnce an I2C bus is available, try detecting devices on it:\n\n```elixir\niex> Circuits.I2C.detect_devices()\nCircuits.I2C.detect_devices\nDevices on I2C bus \"i2c-1\":\n * 64  (0x40)\n * 112 (0x70)\n\n2 devices detected on 1 I2C buses\n```\n\nThe return value here is a list of device addresses that were detected. It is\nstill possible that the device will work even if it does not detect, but you\nprobably want to check wires at this point. If you have a logic analyzer, use it\nto verify that I2C transactions are being initiated on the bus.","ref":"readme.html#how-do-i-debug"},{"type":"extras","title":"I2C seems slow. What could be wrong? - Circuits.I2C","doc":"I2C buses are usually run at 100 kbit/s or 400 kbit/s. Many devices support\nhigher speeds. The tradeoff is that higher speeds are sometimes don't work as\nwell especially if you're using jumper cables to connect parts together. The\nRaspberry Pi runs the I2C bus at a low speed - probably for this reason.\n\nOther things to check:\n\n* Can you reduce the reads and writes? I2C devices let you read or write many\n  bytes at the same time. Each transaction has overhead so minimizing\n  transaction helps.\n* Can you reduce the total number of bytes in each transaction? For example, do\n  you need to read a particular register? Is there a mode that the device can be\n  put it so that it only returns useful data?\n* Can a write and read be combined? The `Circuits.I2C.write_read` function is\n  more efficient than a separate write followed by a read.\n* Does the device support a queue mode? Some devices have internal queues that\n  allow the host to copy out more than one sample each time.","ref":"readme.html#i2c-seems-slow-what-could-be-wrong"},{"type":"extras","title":"Where can I get help? - Circuits.I2C","doc":"The hardest part is communicating with a device for the first time. The issue is\nusually unrelated to `Circuits.I2C`. If you expand your searches to include\nPython and C forums, you'll frequently find the answer.\n\nIf that fails, try posting a question to the [Elixir\nForum](https://elixirforum.com/). Tag the question with `Nerves` and it will\nhave a good chance of getting to the right people. Feel free to do this even if\nyou're not using Nerves.","ref":"readme.html#where-can-i-get-help"},{"type":"extras","title":"Can I develop code that uses Circuits.I2C on my laptop? - Circuits.I2C","doc":"You have a few options:\n\n1. Connect your I2C devices to a USB->I2C adapter like a [Adafruit FT232H\n   Breakout](https://www.adafruit.com/product/2264)\n2. Use the CircuitsSim backend\n3. Create a custom backend and use it to mock interactions with the Circuits.I2C\n   API","ref":"readme.html#can-i-develop-code-that-uses-circuits-i2c-on-my-laptop"},{"type":"extras","title":"Will it run on Arduino? - Circuits.I2C","doc":"No. This only runs on Linux-based boards. If you're interested in controlling an\nArduino from a computer that can run Elixir, check out\n[circuits_uart](https://hex.pm/packages/circuits_uart) for communicating via the\nArduino's serial connection or\n[firmata](https://github.com/mobileoverlord/firmata) for communication using the\nArduino's Firmata protocol.","ref":"readme.html#will-it-run-on-arduino"},{"type":"extras","title":"License - Circuits.I2C","doc":"All original source code in this project is licensed under Apache-2.0.\n\nAdditionally, this project follows the [REUSE recommendations](https://reuse.software)\nand labels so that licensing and copyright are clear at the file level.\n\nExceptions to Apache-2.0 licensing are:\n\n* Linux header files included for convenience under GPL-2.0-or-later WITH\n  Linux-syscall-note\n* Configuration and data files are licensed under CC0-1.0\n* Documentation files are CC-BY-4.0\n* Erlang Embedded board images are Solderpad Hardware License v0.51.","ref":"readme.html#license"},{"type":"extras","title":"Porting","doc":"# Porting","ref":"porting.html"},{"type":"extras","title":"Upgrading Circuits.I2C 1.0 projects to 2.0 - Porting","doc":"Circuits.I2C 2.0 supports alternative I2C hardware and the ability to mock or\nemulate devices via backends. The Linux i2c-dev backend is the default and this\nmatches Circuits.I2C 1.0. Most projects won't need any changes other than to\nupdate the dependency in `mix.exs`. If upgrading a library, The following\ndependency specification is recommended to allow both `circuits_i2c` versions:\n\n```elixir\n   {:circuits_i2c, \"~> 2.0 or ~> 1.0\"}\n```\n\nThe following potentially breaking changes were made:\n\n1. `Circuits.I2C.open/1` no longer accepts Erlang strings.\n2. The `stub` implementation has been renamed to `i2c_dev_test`. If using the\n   stub implementation for testing, you may have to update your tests since\n   there were minor changes.","ref":"porting.html#upgrading-circuits-i2c-1-0-projects-to-2-0"},{"type":"extras","title":"Upgrading Elixir/ALE projects to Circuits.I2C - Porting","doc":"The `Circuits.I2C` package is the next version of Elixir/ALE's I2C support.\nIf you're currently using Elixir/ALE, you're encouraged to switch. Here are some\nbenefits:\n\n1. Supported by both the maintainer of Elixir/ALE and a couple others. They'd\n   prefer to support `Circuits.I2C` issues.\n2. Much faster than Elixir/ALE.\n3. Simplified API\n\n`Circuits.I2C` uses Erlang's NIF interface. NIFs have the downside of being able\nto crash the Erlang VM. Experience with Elixir/ALE has given many of us\nconfidence that this won't be a problem.","ref":"porting.html#upgrading-elixir-ale-projects-to-circuits-i2c"},{"type":"extras","title":"Code modifications - Porting","doc":"`Circuits.I2C` is not a `GenServer`, so if you've added `ElixirALE.I2C` to a\nsupervision tree, you'll have to take it out and manually call\n`Circuits.I2C.open` to obtain a reference. A common pattern is to create a\n`GenServer` that is descriptive of what the I2C device does and have it be\nresponsible for all I2C calls.\n\nThe remain modifications should mostly be mechanical:\n\n1. Rename references to `ElixirALE.I2C` to `Circuits.I2C` and `elixir_ale`\n   to `circuits_i2c`\n2. Change calls to `ElixirALE.I2C.start_link/2` to `Circuits.I2C.open/1`. You'll\n   need to remove the I2C address from the call to open. While you're at it,\n   review the arguments to open to not include any `GenServer` options.\n3. Add the I2C device's bus address to all of the `read`, `write`, and\n   `write_read` calls. We recommend making a short helper function that has\n   the I2C address.\n4. The `read` and `write_read` functions now return `{:ok, result}` tuples on\n   success so add code to handle that. Alternately, call `read!` or `write_read!`\n   and they will raise an exception if there's an error.\n5. Look for calls to `I2C.read_device`, `I2C.write_device` and\n   `I2C.write_read_device` and remove the `_device` part.\n6. Consider adding a call to `Circuits.I2C.close/1` if there's an obvious place\n   to release the I2C. This is not strictly necessary since the garbage\n   collector will free unreferenced I2C references.\n7. If you manually implemented I2C bus retry logic, consider specifying the\n   `:retries` option to have `Circuits.I2C` retry for you.\n8. Change calls to `ElixirALE.I2C.device_names/0` to `Circuits.I2C.bus_names/0`.\n\nIf you find that you have to make any other changes, please let us know via an\nissue or PR so that other users can benefit.","ref":"porting.html#code-modifications"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v2.1.0 - 2025-02-10 - Changelog","doc":"* New features\n  * Support setting a timeout on I2C bus transactions. Support for this depends\n    on the backend and drivers, but it's at least possible to set it. The\n    default is usually 1 second with Linux.\n\n* Improvements\n  * Various documentation and spec cleanup","ref":"changelog.html#v2-1-0-2025-02-10"},{"type":"extras","title":"v2.0.7 - 2025-02-02 - Changelog","doc":"* Improvements\n  * Raise earlier when bad values are passed for the `:retries` option\n  * Update copyrights and license info for REUSE compliance","ref":"changelog.html#v2-0-7-2025-02-02"},{"type":"extras","title":"v2.0.6 - 2024-09-27 - Changelog","doc":"* Bug fixes\n  * Don't crash on ENXIO (errno 6) failures due to unresponsives I2C targets\n  * Add ENXIO as a known error for easier debugging next time","ref":"changelog.html#v2-0-6-2024-09-27"},{"type":"extras","title":"v2.0.5 - 2024-09-08 - Changelog","doc":"Only Elixir 1.13 and later are supported now. There are no known issues with\nusing earlier versions at the moment, but it's no longer regularly checked in\nCI.\n\n* Bug fixes\n  * Fix compilation when Erlang has been installed to a directory with spaces","ref":"changelog.html#v2-0-5-2024-09-08"},{"type":"extras","title":"v2.0.4 - 2024-04-20 - Changelog","doc":"* Improvements\n  * Improve target detection so that setting `MIX_TARGET` when not\n    crosscompiling does the expected thing on MacOS.","ref":"changelog.html#v2-0-4-2024-04-20"},{"type":"extras","title":"v2.0.3 - 2024-01-18 - Changelog","doc":"* Bug fixes\n  * Remove lazy NIF loading. There's an unexplained segfault in a small example\n    program that uses the same strategy. Even though it wasn't reproduceable\n    here, it's not worth the risk. Thanks to @pojiro for investigating.","ref":"changelog.html#v2-0-3-2024-01-18"},{"type":"extras","title":"v2.0.2 - 2024-01-14 - Changelog","doc":"* Changes\n  * Fix race condition when multiple processes load the NIF simultaneously that\n    would cause an unnecessary crash.\n  * Bump minimum supported version of Elixir to 1.11 since no longer testing\n    with it due to a dependency issue.","ref":"changelog.html#v2-0-2-2024-01-14"},{"type":"extras","title":"v2.0.1 - 2023-10-22 - Changelog","doc":"* Changes\n  * Add `Circuits.I2C.bus()` type back to fix a dialyzer warning on projects\n    that support both Circuits.I2C 1.0 and 2.0.","ref":"changelog.html#v2-0-1-2023-10-22"},{"type":"extras","title":"v2.0.0 - 2023-08-14 - Changelog","doc":"This is a major update to Circuits.I2C that removes the requirement to use\nNerves or Linux. The API is almost the same and the default is to compile and\nuse the Linux backend, so changes may not be needed.\n\n* Changes\n  * Support alternative backends for different operating systems or for\n    simulated hardware\n  * For the Linux NIF, support passing a `:retry` count to `Circuits.I2C.open/2`\n    that will apply to all I2C operations. This simplifies dealing with devices\n    with flaky connections.\n  * Defer loading the Linux NIF until `Circuits.I2C.open/2` is called\n  * Return error on `Circuits.I2C.open/2` if the device isn't a real I2C device.\n    This was previously detected on the first operation.\n  * For the Linux NIF, return errno numbers rather than their `strerr` strings\n    when they're unknown. It turned out that the strings could be more confusing\n    that the numbers.","ref":"changelog.html#v2-0-0-2023-08-14"},{"type":"extras","title":"v1.2.2 - 2023-03-24 - Changelog","doc":"* Fixes\n  * Add types.h compatibility header to hex package so that host MacOS builds\n    work again.","ref":"changelog.html#v1-2-2-2023-03-24"},{"type":"extras","title":"v1.2.1 - 2023-03-20 - Changelog","doc":"* Fixes\n  * Detect I2C controllers that don't support 0-byte writes and revert to the\n    old detection heuristic. This fixes an issue on Beaglebones (AM335x) that\n    caused devices to be missed and kernel warnings to be logged.","ref":"changelog.html#v1-2-1-2023-03-20"},{"type":"extras","title":"v1.2.0 - 2023-03-17 - Changelog","doc":"* Changes\n  * Improve device detection by using 0-byte writes on some I2C addresses and\n    1-byte reads on others. This matches the i2c-tools heuristic and detects at\n    least on more device that wasn't detected before.\n  * Simplified NIF by deleting a lot of flexibility that didn't end up being\n    useful. Also moved functionality around so that it could be implemented more\n    simply.","ref":"changelog.html#v1-2-0-2023-03-17"},{"type":"extras","title":"v1.1.0 - 2022-11-16 - Changelog","doc":"* Changes\n  * Immediately close I2C bus references after discovery. Waiting for the GC to\n    collect them could cause intermittent failures in rare scenarios where\n    multiple I2C device discoveries are done close together. This likely only\n    affects CI in practice.\n  * Remove Erlang convenience functions since no one used them\n  * Require Elixir 1.10 or later. Previous versions probably work, but won't be\n    supported. This opens up the possibility of using Elixir 1.10+ features in\n    future releases.","ref":"changelog.html#v1-1-0-2022-11-16"},{"type":"extras","title":"v1.0.1 - 2021-12-28 - Changelog","doc":"* Fixes\n  * Properly mark I/O bound functions in NIF.","ref":"changelog.html#v1-0-1-2021-12-28"},{"type":"extras","title":"v1.0.0 - 2021-10-20 - Changelog","doc":"This release only changes the version number. No code has changed.","ref":"changelog.html#v1-0-0-2021-10-20"},{"type":"extras","title":"v0.3.9 - Changelog","doc":"This release only has doc and build output cleanup. No code has changed.","ref":"changelog.html#v0-3-9"},{"type":"extras","title":"v0.3.8 - Changelog","doc":"* New features\n  * Add `Circuits.I2C.discover/2` and `Circuits.I2C.discover_one/2`. These\n    functions are intended for library authors wanting to provide good\n    suggestions or defaults to their users. See the hex docs for more\n    information. Thanks to Bruce Tate for the idea and PR.\n\n* Improvements\n  * The stub I2C implementation is now used whenever `MIX_ENV=test`. While this\n    is not generally useful for testing code that uses Circuits.I2C, it does\n    prevent accidental use of real I2C buses in unit tests on those systems\n    with real I2C buses.","ref":"changelog.html#v0-3-8"},{"type":"extras","title":"v0.3.7 - Changelog","doc":"* Improvements\n  * Add I2C address in hex showing detected devices","ref":"changelog.html#v0-3-7"},{"type":"extras","title":"v0.3.6 - Changelog","doc":"* Bug fixes\n  * Add -fPIC to compilation flags to fix build with `nerves_system_x86_64` and\n    other environments using the Musl C toolchains","ref":"changelog.html#v0-3-6"},{"type":"extras","title":"v0.3.5 - Changelog","doc":"* Bug fixes\n  * Reduce the number of I2C addresses scanned for detection to avoid confusing\n    some devices.","ref":"changelog.html#v0-3-5"},{"type":"extras","title":"v0.3.4 - Changelog","doc":"This release should work on Erlang/OTP 20 - 22 and Elixir 1.4 and\nnewer. The CI process has been updated to verify more versions now.\n\n* Bug fixes\n  * Improve error message when bus doesn't exist","ref":"changelog.html#v0-3-4"},{"type":"extras","title":"v0.3.3 - Changelog","doc":"* Bug fixes\n  * Fix binary handling in NIF. This fixes segfaults and other errors when run\n    on Raspbian.","ref":"changelog.html#v0-3-3"},{"type":"extras","title":"v0.3.2 - Changelog","doc":"* Bug fixes\n  * Fix file handle leak when I2C bus references were garbage collected.","ref":"changelog.html#v0-3-2"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"* Bug fixes\n  * Build C source under the `_build` directory so that changing targets\n    properly rebuilds the C code as well as the Elixir code.","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"Print detected devices instead of an error.","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"Minor text updates.\n\nRemove i2c_ from i2c_address and i2c_bus.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.0"}}